import AlteryxPythonSDK as Sdk
import engine.sf_metadata as sf_metadata
import engine.sf_utils as sf_utils
import json
import pickle

def push_records(self, header_list: list, record_values: list):
  # Store metadata in temp file
  if self.query_type == 'existingReport':
    file_path = self.alteryx_engine.create_temp_file_name(extension=str(self.temp_file_name), options=2)
    with open(file_path, 'wb') as fp:
      pickle.dump(header_list, fp)

    self.record_info_out = sf_metadata.build_record_info_out(self=self, response_object=header_list)
    self.output_anchor.init(self.record_info_out)  # Lets the downstream tools know of the outgoing record metadata.
  
  record_creator = self.record_info_out.construct_record_creator()  # Creating a new record_creator for the new data.

  for record in record_values:
    for field in enumerate(record):
      if field[1] is None:
        self.record_info_out[field[0]].set_null(record_creator)
      else:
        if type(field[1]) == dict:
          self.record_info_out[field[0]].set_from_string(record_creator, str(json.dumps(field[1])))
        else:
          self.record_info_out[field[0]].set_from_string(record_creator, str(field[1]))
    out_record = record_creator.finalize_record()
    self.output_anchor.push_record(out_record, False)  # False: completed connections will automatically close.

    self.record_count += 1
    progress_percentage = self.record_count / self.total_records
    if (progress_percentage * 100) % 10 == 0:
      self.output_anchor.update_progress(progress_percentage)

    record_creator.reset()  # Resets the variable length data to 0 bytes (default) to prevent unexpected results.
  return

def push_raw_json(self, response: dict):
  response_object = [{'name': 'JSON', 'type': Sdk.FieldType.v_wstring, 'length': 1073741823, 'scale': 0, 'label': 'JSON'}]
  response = json.dumps(response)
  # Store metadata in temp file  
  file_path = self.alteryx_engine.create_temp_file_name(extension=str(self.temp_file_name), options=2)
  with open(file_path, 'wb') as fp:
    pickle.dump(response_object, fp)
  
  record_info_out = sf_metadata.build_record_info_out(self=self, response_object=response_object)  # Building out the outgoing record layout.
  self.output_anchor.init(record_info_out)  # Lets the downstream tools know of the outgoing record metadata.
  record_creator = record_info_out.construct_record_creator()  # Creating a new record_creator for the new data.
  record_info_out[0].set_from_string(record_creator, str(response))
  # Asking for a record to push downstream
  out_record = record_creator.finalize_record()
  self.output_anchor.push_record(out_record, False)  # False: completed connections will automatically close.

  self.record_count += 1
  progress_percentage = self.record_count / self.total_records
  if (progress_percentage * 100) % 10 == 0:
    self.output_anchor.update_progress(progress_percentage)

  record_creator.reset()  # Resets the variable length data to 0 bytes (default) to prevent unexpected results.
  self.output_anchor.close()  # Close outgoing connections.
  return
