import AlteryxPythonSDK as Sdk
import engine.sf_utils as sf_utils
import requests

def build_record_info_out(self, response_object: list):
  """
  A non-interface helper for pi_push_all_records() responsible for creating the outgoing record layout.
  :param response_object: The name for the json response.
  :return: The outgoing record layout.
  """

  record_info_out = Sdk.RecordInfo(self.alteryx_engine)  # A fresh record info object for outgoing records.
  for field in response_object:
    record_info_out.add_field(field['name'], field['type'], field['length'], field['scale'], 'Salesforce', field['label']) 

  return record_info_out      
    
def find_field_name(field_name: str, metadata_list: list):
  match = ''
  for item in metadata_list['fields']:
    if item['name'] == field_name:
      match = item
    
  if match:
    return match
  else:
    return {'name': field_name, 'type': Sdk.FieldType.v_wstring, 'length': 254, 'scale': 0, 'label': field_name}

def retrieve_metadata(self, table_name: str, field_names: list):
  describe_endpoint = self.instance_url + '/services/data/' + self.api_version + '/sobjects/' + table_name + '/describe/'
  payload = {'Authorization' : 'Bearer ' + self.access_token}

  try: 
    metadata_response = requests.get(describe_endpoint, headers = payload)
    metadata_response.raise_for_status() 
    metadata_response = metadata_response.json()
    self.metadata_list = [find_field_name(field_name, metadata_response) for field_name in field_names]
    return self.metadata_list
      
  except requests.exceptions.HTTPError as error: 
    sf_utils.display_msg(self=self, type='error', msg_string=str(error))
    
def map_field_types(metadata_list: list):
  for i in range(len(metadata_list)):
    if metadata_list[i]['type'] == 'currency' or metadata_list[i]['type'] == 'double':
      metadata_list[i]['type'] = Sdk.FieldType.double
      metadata_list[i]['length'] = 8
    elif metadata_list[i]['type'] == 'int':
      metadata_list[i]['type'] = Sdk.FieldType.int32
      metadata_list[i]['length'] = 4
    elif metadata_list[i]['type'] == 'boolean':
      metadata_list[i]['type'] = Sdk.FieldType.bool
      metadata_list[i]['length'] = 1
    elif metadata_list[i]['type'] == 'date':
      metadata_list[i]['type'] = Sdk.FieldType.date
    elif metadata_list[i]['type'] == 'datetime':
      metadata_list[i]['type'] = Sdk.FieldType.datetime
    else:
      metadata_list[i]['type'] = Sdk.FieldType.v_wstring
      metadata_list[i]['length'] = metadata_list[i]['length'] if metadata_list[i]['length'] != 0 else 131072
  return metadata_list