import AlteryxPythonSDK as Sdk
import engine.sf_datapush as sf_datapush
import engine.sf_metadata as sf_metadata
import engine.sf_utils as sf_utils
import aiohttp
import asyncio
import json
import urllib.parse
from flatten_dict import flatten


async def retrieve_query_json_chunk_parse_response(self, session, endpoint, header):
  async with session.get(endpoint, headers = header) as response:
    response.raise_for_status()
    response = await response.json()
    self.total_records = response["totalSize"]

    if not response['records']:
      record_reader=[]
    else:
      record_reader = parse_query_response(self=self, json_response=response, field_map=self.header_list)

    sf_datapush.push_records(self=self, header_list=self.header_list, record_values=record_reader)
  return response

async def retrieve_query_json_chunk_parse_json(self, session, endpoint, header):
  async with session.get(endpoint, headers = header) as response:
    response.raise_for_status()
    response = await response.json()
    self.total_records = response["totalSize"]

    if not response['records']:
      record_reader=[]
    else:
      record_reader = parse_query_json(self=self, json_response=response)

    sf_datapush.push_records(self=self, header_list=self.header_list, record_values=record_reader)
  return response

async def retrieve_query_json(self, endpoint, header):
  async with aiohttp.ClientSession() as session:
    if self.json_parse == 'True' and self.query_type == 'customQuery' and self.parse_query_flag == 'output_json':
      self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.info, self.alteryx_engine.xmsg("Unable to parse records into table. Results output as JSON."))
      response = await retrieve_query_json_chunk_parse_json(self, session, endpoint, header)
    
      while not response['done']:
        endpoint = self.instance_url + response['nextRecordsUrl'] 
        response = await retrieve_query_json_chunk_parse_json(self, session, endpoint, header)

    elif self.json_parse == 'False' and self.query_type == 'customQuery':
      response = await retrieve_query_json_chunk_parse_json(self, session, endpoint, header)

      while not response['done']:
        endpoint = self.instance_url + response['nextRecordsUrl'] 
        response = await retrieve_query_json_chunk_parse_json(self, session, endpoint, header)

    else:
      response = await retrieve_query_json_chunk_parse_response(self, session, endpoint, header)
    
      while not response['done']:
        endpoint = self.instance_url + response['nextRecordsUrl'] 
        response = await retrieve_query_json_chunk_parse_response(self, session, endpoint, header)

def process_query(self):
  self.records = None
  self.record_count = 0

  if self.query:
    encoded_query = urllib.parse.quote_plus(self.query) 
    query_endpoint = self.instance_url + '/services/data/' + self.api_version + '/queryAll/?q=' + encoded_query 
    header = {'Authorization' : 'Bearer ' + self.access_token}

    loop = asyncio.get_event_loop()
    loop.run_until_complete(retrieve_query_json(self, query_endpoint, header))
  
    self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.info, self.alteryx_engine.xmsg('@1 records were read from Salesforce.', str(self.total_records)))
    self.output_anchor.close()  # Close outgoing connections.

  return

def parse_query_json(self, json_response: dict):
  def update_item(item):
    if (item['attributes']['type'] == 'Attachment' or item['attributes']['type'] == 'Document') and 'Body' in item.keys():
      item['Body'] = item['attributes']['url'] + '/Body'
      return [json.dumps(item)]

    return [json.dumps(item)]

  return [update_item(item) for item in json_response['records']]

def parse_query_response(self, json_response: list, field_map: list):
  datetime_fields = [item['name'] for item in field_map if item['type'] == Sdk.FieldType.datetime]
  def update_item(item):
    if (item['attributes']['type'] == 'Attachment' or item['attributes']['type'] == 'Document') and 'Body' in item.keys():
      item['Body'] = item['attributes']['url'] + '/Body'
    for k,v in item.items():
      if k in datetime_fields and v:
        item[k] = v[0:19].replace("T", " ")
    if self.query_type == 'customQuery' and self.parse_query_flag == 'dot_query':
      item = flatten(item, reducer=dict_reducer)
      item = {k:v for (k,v) in item.items() if 'attributes.url' not in k if 'attributes.type' not in k}
      return item
    else:
      del item['attributes']
      return item

  return [list(record.values()) for record in [update_item(item) for item in json_response['records']]]
  
def dict_reducer(k1, k2):
  if k1 is None:
    return k2
  else:
    return k1 + '.' + k2