import AlteryxPythonSDK as Sdk
import engine.sf_datapush as sf_datapush
import engine.sf_metadata as sf_metadata
import engine.sf_utils as sf_utils
import requests
from time import sleep

def process_report(self):
  if self.report_id == None:
    self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.error, self.alteryx_engine.xmsg("Select a report"))
  else:
    execute_report_run(self=self)   
  return

def execute_report_run(self):
  report_endpoint = self.instance_url + '/services/data/' + self.api_version + '/analytics/reports/' + self.report_id + '/instances' 
  payload = {'Authorization' : 'Bearer ' + self.access_token}

  try: 
    response = requests.post(report_endpoint, headers=payload) 
    response.raise_for_status() 
    response = response.json()
    report_results_id = response["id"]
    pull_report_data(self=self, report_results_id=report_results_id)
    return

  except requests.exceptions.RequestException as error: 
    sf_utils.display_msg(self=self, type='error', msg_string=str(error))
    return

def pull_report_data(self, report_results_id: str):
  report_data_endpoint = self.instance_url + '/services/data/' + self.api_version + '/analytics/reports/' + self.report_id + '/instances/' + report_results_id
  payload = {'Authorization' : 'Bearer ' + self.access_token}
  sleep(2) # First call is "New" status without it
  try: 
    response = requests.get(report_data_endpoint, headers=payload) 
    response.raise_for_status() 
    response = response.json()
    report_status = response["attributes"]["status"]

    if report_status == 'Success':
      parse_report_response(self=self, report_response=response)
      return
    elif report_status == 'Error':
      self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.error, self.alteryx_engine.xmsg("Report had an unknown failure. Confirm report can be run within Salesforce."))
      return
    else:
      self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.info, self.alteryx_engine.xmsg('Report is @1. Retrying in five seconds.', str(report_status)))
      # Wait five seconds before requesting report data again
      sleep(5)
      pull_report_data(self=self, report_results_id=report_results_id)
          
  except requests.exceptions.RequestException as error: 
    sf_utils.display_msg(self=self, type='error', msg_string=str(error))
    return

def parse_report_response(self, report_response: str):
  try:
    report_type = report_response["reportMetadata"]["reportFormat"]
    
    # Parsing flow for TABULAR reports when report_parse is True
    if report_type == 'TABULAR' and self.report_parse == 'True':
      # Determine field names
      field_dict = report_response["reportExtendedMetadata"]["detailColumnInfo"]

      # Determine field data types
      metadata_list = [
          {
            "name": field_dict[k]["label"],
            "label": field_dict[k]["label"],
            "length": 0,
            "scale": 0,
            "type": field_dict[k]["dataType"]
          } 
          for k in field_dict.keys()
        ]

      # Outputting date/datetime as string due to inconsistent formatting from SF
      metadata_list = [{k:v if v != 'date' and v != 'datetime' else 'string' for k,v in item.items()} for item in metadata_list]
      header_list = sf_metadata.map_field_types(metadata_list=metadata_list)

      # Determine record values
      record_list = report_response["factMap"]["T!T"]["rows"]
      records = [item["dataCells"] for item in record_list]
      record_values = []
      for item in records:
        item_list = []
        for obj in item:
          try:
            if type(obj["value"]) == str:
              item_list.append(obj["label"])
            elif type(obj["value"]) == dict:
              if 'amount' in obj["value"]:
                item_list.append(obj["value"]["amount"])
              else:
                item_list.append(obj["label"])
            else:
              item_list.append(obj["value"])
          except:
            item_list.append(None)
        record_values.append(item_list)
      self.total_records = len(record_values)
      self.record_count = 0
      sf_datapush.push_records(self=self, header_list=header_list, record_values=record_values)
    # Report isn't TABULAR or report_parse isn't true then output as raw json
    else:
      if self.report_parse == 'True':
        self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.info, self.alteryx_engine.xmsg("Unable to parse report. Report output as JSON."))
      else:
        self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.info, self.alteryx_engine.xmsg("Attempt to parse report results not selected. Report output as JSON."))
      self.total_records = len(report_response)
      self.record_count = 0
      sf_datapush.push_raw_json(self=self, response=report_response)
  except:
    self.alteryx_engine.output_message(self.n_tool_id, Sdk.EngineMessageType.info, self.alteryx_engine.xmsg("Unable to parse report. Report output as JSON."))
    self.total_records = len(report_response)
    self.record_count = 0
    sf_datapush.push_raw_json(self=self, response=report_response)
  return
